/*
 * MiTCR <http://milaboratory.com>
 *
 * Copyright (c) 2010-2013:
 *     Bolotin Dmitriy     <bolotin.dmitriy@gmail.com>
 *     Chudakov Dmitriy    <chudakovdm@mail.ru>
 *
 * MiTCR is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.milaboratory.core.sequence.nucleotide;

import com.milaboratory.core.sequence.Sequence;
import com.milaboratory.util.Bit2Array;

import java.io.Serializable;

/**
 * Main implementation of nucleotide sequence.
 *
 * @author Bolotin Dmitriy (bolotin.dmitriy@gmail.com)
 * @author Shugay Mikhail (mikhail.shugay@gmail.com)
 */
public class NucleotideSequenceImpl extends NucleotideSequence implements Serializable {
    private static final long serialVersionUID = 1L;

    private Bit2Array data;

    protected NucleotideSequenceImpl(Bit2Array data) {
        this.data = data;
    }

    @Override
    public byte codeAt(int position) {
        return (byte) data.get(position);
    }

    @Override
    public Sequence clone() {
        return new NucleotideSequenceImpl(data.clone());
    }

    @Override
    public int size() {
        return data.size();
    }

    public Bit2Array getInnerStorage() {
        return data;
    }

    /*@Override
    public boolean equals(Object obj) {
        if (obj == null)
            return false;
        if (!(obj instanceof NucleotideSequenceImpl))
            return false;
        final NucleotideSequenceImpl other = (NucleotideSequenceImpl) obj;
        if (!this.data.equals(other.data))
            return false;
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 5;
        hash = 83 * hash + this.data.hashCode();
        return hash;
    }*/

    public static NucleotideSequenceImpl fromStorage(Bit2Array b2a) {
        return new NucleotideSequenceImpl(b2a);
    }

    public static NucleotideSequenceImpl fromSequence(char[] sequence) {
        Bit2Array storage = new Bit2Array(sequence.length);
        byte code;
        for (int i = 0; i < sequence.length; ++i) {
            code = NucleotideAlphabet.INSTANCE.codeFromSymbol(sequence[i]);
            if (code > 3)
                throw new RuntimeException("This sequence doesn't support 'N/n' nucleotides.");
            storage.set(i, code);
        }
        return new NucleotideSequenceImpl(storage);
    }

    public static NucleotideSequenceImpl fromSequence(String sequence) {
        return fromSequence(sequence.toCharArray());
    }

    public static NucleotideSequenceImpl fromSequence(NucleotideSequence sequence) {
        return new NucleotideSequenceImpl(storageFromSequence(sequence));
    }

    public static Bit2Array storageFromSequence(NucleotideSequence sequence) {
        if (sequence instanceof NucleotideSequenceImpl)
            return ((NucleotideSequenceImpl) sequence).data.clone();
        Bit2Array storage = new Bit2Array(sequence.size());
        for (int i = 0; i < sequence.size(); ++i)
            storage.set(i, sequence.codeAt(i));
        return storage;
    }

    public static NucleotideSequenceImpl fromSequence(byte[] sequence, int offset, int length) {
        Bit2Array storage = new Bit2Array(length);
        for (int i = 0; i < length; ++i)
            storage.set(i, NucleotideAlphabet.INSTANCE.codeFromSymbol((char) sequence[offset + i]));
        return new NucleotideSequenceImpl(storage);
    }

    public int toOriginalCoord(int proxyCoord) {
        return proxyCoord;
    }

    @Override
    public NucleotideSequence getOriginal() {
        return this;
    }
}
