/*
 * MiTCR <http://milaboratory.com>
 *
 * Copyright (c) 2010-2013:
 *     Bolotin Dmitriy     <bolotin.dmitriy@gmail.com>
 *     Chudakov Dmitriy    <chudakovdm@mail.ru>
 *
 * MiTCR is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.milaboratory.core.sequence.util;

import com.milaboratory.core.sequence.nucleotide.NucleotideSequence;
import com.milaboratory.core.sequence.nucleotide.NucleotideSequenceImpl;
import com.milaboratory.util.Bit2Array;

/**
 * Nucleotide reverse complement sequence.
 *
 * @author Bolotin Dmitriy (bolotin.dmitriy@gmail.com)
 * @author Shugay Mikhail (mikhail.shugay@gmail.com)
 */
public class NucleotideRCSequence extends NucleotideSequenceImpl {
    private NucleotideSequence originalSequence;

    public NucleotideRCSequence(NucleotideSequence originalSequence) {
        super(transformNucleotideStorage(originalSequence));
        this.originalSequence = originalSequence;
    }

    @Override
    public int toOriginalCoord(int proxyCoord) {
        return originalSequence.toOriginalCoord(originalSequence.size() - 1 - proxyCoord);
    }

    @Override
    public NucleotideSequence getOriginal() {
        return originalSequence.getOriginal();
    }

    private static Bit2Array transformNucleotideStorage(NucleotideSequence sequence) {
        Bit2Array newData = new Bit2Array(sequence.size());
        int reverseCoord;
        for (int coord = 0; coord < sequence.size(); coord++) {
            reverseCoord = sequence.size() - 1 - coord;
            newData.set(coord, (~sequence.codeAt(reverseCoord)) & 0x3);
        }
        return newData;
    }
}
