/*
 * MiTCR <http://milaboratory.com>
 *
 * Copyright (c) 2010-2013:
 *     Bolotin Dmitriy     <bolotin.dmitriy@gmail.com>
 *     Chudakov Dmitriy    <chudakovdm@mail.ru>
 *
 * MiTCR is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.milaboratory.core.sequence.util;

import com.milaboratory.core.sequence.SequenceQuality;
import com.milaboratory.core.sequence.quality.SequenceQualityPhred;

import java.util.Arrays;

/**
 * Sub sequence quality object.
 *
 * @author Bolotin Dmitriy (bolotin.dmitriy@gmail.com)
 * @author Shugay Mikhail (mikhail.shugay@gmail.com)
 */
public class SubSequenceQuality implements SequenceQuality {
    private SequenceQuality quality;
    private int from, length;

    public SubSequenceQuality(SequenceQuality quality, int from, int length) {
        this.quality = quality;
        this.from = from;
        this.length = length;
    }

    @Override
    public byte[] encode(byte offset) {
        throw new UnsupportedOperationException();
    }

    @Override
    public byte rawQualityValue(int coord) {
        if (coord < 0 || coord >= length)
            throw new IndexOutOfBoundsException();
        return quality.rawQualityValue(coord + from);
    }

    @Override
    public byte minRawQuality() {
        byte min = Byte.MAX_VALUE;
        for (int i = 0; i < length; ++i)
            if (quality.rawQualityValue(i + from) < min)
                min = quality.rawQualityValue(i + from);
        return min;
    }

    @Override
    public float probabilityOfErrorAt(int coord) {
        if (coord < 0 || coord >= length)
            throw new IndexOutOfBoundsException();
        return quality.probabilityOfErrorAt(coord + from);
    }

    @Override
    public float log10ProbabilityOfErrorAt(int coord) {
        if (coord < 0 || coord >= length)
            throw new IndexOutOfBoundsException();
        return quality.log10ProbabilityOfErrorAt(coord + from);
    }

    public SequenceQuality getInnerSequenceQuality() {
        return quality;
    }

    public int getFrom() {
        return from;
    }

    @Override
    public void mergeWith(SequenceQuality quality) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public void setRawQualityValue(int coord, byte value) {
        throw new UnsupportedOperationException("Not supported yet.");
    }

    @Override
    public Class<? extends SequenceQuality> getBaseClass() {
        return quality.getBaseClass();
    }

    @Override
    public int size() {
        return length;
    }

    public SequenceQuality toNativeQualityObject() {
        if (quality instanceof SequenceQualityPhred)
            return new SequenceQualityPhred(Arrays.copyOfRange(((SequenceQualityPhred) quality).getRawData(), from, from + length));
        throw new UnsupportedOperationException();
    }

    @Override
    public SequenceQuality reverse() {
        return new SubSequenceQuality(quality.reverse(), quality.size() - from - length, length);
    }
}
