/*
 * MiTCR <http://milaboratory.com>
 *
 * Copyright (c) 2010-2013:
 *     Bolotin Dmitriy     <bolotin.dmitriy@gmail.com>
 *     Chudakov Dmitriy    <chudakovdm@mail.ru>
 *
 * MiTCR is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.milaboratory.core.sequencing.io.fastq;

import cc.redberry.pipe.Processor;
import com.milaboratory.core.sequence.NucleotideSQPair;
import com.milaboratory.core.sequence.SequenceQuality;
import com.milaboratory.core.sequencing.io.fastq.quality.QualityStringFormat;

/**
 * Trims bad quality nucleotides from {@link NucleotideSQPair}s
 *
 * @author Bolotin Dmitriy (bolotin.dmitriy@gmail.com)
 */
public class NucleotideSQPairTrimmer implements Processor<NucleotideSQPair, NucleotideSQPair> {
    private QualityStringFormat format;

    /**
     * Creates bad quality trimmer
     *
     * @param format sequencing quality format
     */
    public NucleotideSQPairTrimmer(QualityStringFormat format) {
        this.format = format;
    }

    @Override
    public NucleotideSQPair process(NucleotideSQPair input) {
        SequenceQuality quality = input.getQuality();
        int trimTo = quality.size() - 1;
        for (int i = quality.size() - 1; i >= 0; --i)
            if (quality.rawQualityValue(i) != format.getZeroQualityValue())
                break;
            else
                trimTo = i;
        return input.getSubPairCopy(0, trimTo).getDeepCopy();
    }
}
