/*
 * MiTCR <http://milaboratory.com>
 *
 * Copyright (c) 2010-2013:
 *     Bolotin Dmitriy     <bolotin.dmitriy@gmail.com>
 *     Chudakov Dmitriy    <chudakovdm@mail.ru>
 *
 * MiTCR is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.milaboratory.core.sequencing.io.fastq.quality;

/**
 * A base class to store various sequencing quality formats
 */
public enum QualityStringFormat {
    Sanger((byte) 33, (byte) 0, (byte) 40, false, new PhredSequenceQualityFactory(), "Sanger", "sanger"),
    //Solexa((byte) 64, (byte) -5, (byte) 40, false, new SolexaSequenceQualityFactory(), "Illumina 1.2 or less", "illumina12"),
    Illumina13((byte) 64, (byte) 0, (byte) 40, false, new PhredSequenceQualityFactory(), "Illumina 1.3+", "illumina13"),
    Illumina15((byte) 64, (byte) 2, (byte) 40, true, new PhredSequenceQualityFactory(), "Illumina 1.5+", "illumina15"),
    Illumina18((byte) 33, (byte) 2, (byte) 45, true, new PhredSequenceQualityFactory(), "Illumina [Casava 1.8]", "illumina18"),
    Illumina18u((byte) 33, (byte) 2, (byte) 45, true, new PhredSequenceQualityFactory(), "Illumina Unsafe [Casava 1.8]", "illumina18u", true);
    private byte qualityCodeOffset;
    private byte qualityCodeFrom;
    private byte qualityCodeTo;
    private byte zeroQualityValue;
    private boolean zeroFlanked, isUnSafe;
    private SequenceQualityFactory qualityFactory;
    private String name, xmlRepresentation;

    private QualityStringFormat(byte qualityCodeOffset, byte qualityCodeFrom, byte qualityCodeTo, boolean zeroFlanked, AbstractSequenceQualityFactory qualityFactory, String name,
                                String xmlRepresentation) {
        this.qualityCodeOffset = qualityCodeOffset;
        this.qualityCodeFrom = qualityCodeFrom;
        this.qualityCodeTo = qualityCodeTo;
        this.zeroQualityValue = qualityCodeFrom;
        this.zeroFlanked = zeroFlanked;
        this.xmlRepresentation = xmlRepresentation;
        qualityFactory.setQualityCodeFrom(qualityCodeFrom);
        qualityFactory.setQualityCodeOffset(qualityCodeOffset);
        qualityFactory.setQualityCodeTo(qualityCodeTo);
        this.qualityFactory = qualityFactory;
        this.name = name;
        this.isUnSafe = true;
    }

    private QualityStringFormat(byte qualityCodeOffset, byte qualityCodeFrom, byte qualityCodeTo, boolean zeroFlanked, AbstractSequenceQualityFactory qualityFactory, String name,
                                String xmlRepresentation, boolean unSafe) {
        this(qualityCodeOffset, qualityCodeFrom, qualityCodeTo, zeroFlanked, qualityFactory, name, xmlRepresentation);
        qualityFactory.setLowerUnSafe(unSafe);
        this.isUnSafe = unSafe;
    }

    public byte getQualityCodeFrom() {
        return qualityCodeFrom;
    }

    public byte getQualityCodeOffset() {
        return qualityCodeOffset;
    }

    public byte getQualityCodeTo() {
        return qualityCodeTo;
    }

    public SequenceQualityFactory getQualityFactory() {
        return qualityFactory;
    }

    public boolean isZeroFlanked() {
        return zeroFlanked;
    }

    public byte getZeroQualityValue() {
        return zeroQualityValue;
    }

    public String getXmlRepresentation() {
        return xmlRepresentation;
    }

    public boolean isUnSafe() {
        return isUnSafe;
    }

    public static QualityStringFormat fromXML(String xml) {
        for (QualityStringFormat format : values())
            if (format.xmlRepresentation.equals(xml))
                return format;
        return null;
    }

    @Override
    public String toString() {
        return name;
    }
}
