/*
 * MiTCR <http://milaboratory.com>
 *
 * Copyright (c) 2010-2013:
 *     Bolotin Dmitriy     <bolotin.dmitriy@gmail.com>
 *     Chudakov Dmitriy    <chudakovdm@mail.ru>
 *
 * MiTCR is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation, either version 3 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package com.milaboratory.mitcr.cli;

import com.milaboratory.core.clone.Clone;
import com.milaboratory.core.clone.CloneSet;
import com.milaboratory.util.TablePrintStreamAdapter;

import java.io.FileNotFoundException;
import java.io.PrintStream;

public final class TabDelimitedExporter {
    public static final String FULL_MAGIC_PREFIX = "MiTCRFullExportV1.0";

    private TabDelimitedExporter() {
    }

    public static void printHeader(TablePrintStreamAdapter table, ExportDetalizationLevel detalization) {
        switch (detalization) {
            case Full:
                table.row("Read count", "Percentage", "CDR3 nucleotide sequence", "CDR3 nucleotide quality", "Min quality",
                        "CDR3 amino acid sequence", "V alleles", "V segments", "J alleles", "J segments",
                        "D alleles", "D segments", "Last V nucleotide position ", "First D nucleotide position", "Last D nucleotide position",
                        "First J nucleotide position", "VD insertions", "DJ insertions", "Total insertions");
                return;
            case Medium:
                table.row("Read count", "Percentage", "CDR3 nucleotide sequence",
                        "CDR3 amino acid sequence", "V segments", "J segments",
                        "D segments", "Last V nucleotide position ", "First D nucleotide position", "Last D nucleotide position",
                        "First J nucleotide position", "VD insertions", "DJ insertions", "Total insertions");
                return;
            case Simple:
                table.row("Read count", "Percentage", "CDR3 nucleotide sequence",
                        "CDR3 amino acid sequence", "V segments", "J segments",
                        "D segments");
                return;
        }
    }

    public static void printClone(TablePrintStreamAdapter table, Clone clone, ExportDetalizationLevel detalization) {
        switch (detalization) {
            case Full:
                table.row(clone.getCount(), clone.getPart(), clone.getCDR3().getSequence(), clone.getCDR3().getQuality(),
                        clone.getCDR3().getQuality().minRawQuality(),
                        clone.getCDR3AA(), clone.getVAlleles(), clone.getVSegments(), clone.getJAlleles(), clone.getJSegments(),
                        clone.getDAlleles(), clone.getDSegments(), clone.getVEnd(), clone.getDStart(), clone.getDEnd(),
                        clone.getJStart(), clone.insertionsVD(), clone.insertionsDJ(), clone.insertionsTotal());
                return;
            case Medium:
                table.row(clone.getCount(), clone.getPart(), clone.getCDR3().getSequence(),
                        clone.getCDR3AA(), clone.getVSegments(), clone.getJSegments(),
                        clone.getDSegments(), clone.getVEnd(), clone.getDStart(), clone.getDEnd(),
                        clone.getJStart(), clone.insertionsVD(), clone.insertionsDJ(), clone.insertionsTotal());
                return;
            case Simple:
                table.row(clone.getCount(), clone.getPart(), clone.getCDR3().getSequence(),
                        clone.getCDR3AA(), clone.getVSegments(), clone.getJSegments(),
                        clone.getDSegments());
        }
    }

    public static void exportCloneSet(TablePrintStreamAdapter table, CloneSet cloneSet, ExportDetalizationLevel detalization) {
        table.row(FULL_MAGIC_PREFIX, cloneSet.getSpecies().name(), cloneSet.getGene().getXmlRepresentation());
        printHeader(table, detalization);
        for (Clone clone : cloneSet.getClones())
            printClone(table, clone, detalization);
    }

    public static void exportCloneSet(PrintStream stream, CloneSet cloneSet, ExportDetalizationLevel detalization) {
        exportCloneSet(new TablePrintStreamAdapter(stream), cloneSet, detalization);
    }

    public static void exportCloneSet(String fileName, CloneSet cloneSet, ExportDetalizationLevel detalization) throws FileNotFoundException {
        exportCloneSet(new PrintStream(fileName), cloneSet, detalization);
    }
}
